package com.company.tour.view.dashboard;


import com.company.tour.app.VehiclesService;
import com.company.tour.entity.Tour;
import com.company.tour.entity.TourStop;
import com.company.tour.entity.Vehicle;
import com.company.tour.view.main.MainView;

import com.vaadin.flow.router.Route;
import com.vaadin.flow.server.StreamResource;
import io.jmix.core.LoadContext;
import io.jmix.flowui.model.CollectionContainer;
import io.jmix.flowui.model.CollectionLoader;
import io.jmix.flowui.model.InstanceContainer;
import io.jmix.flowui.view.*;
import io.jmix.mapsflowui.component.GeoMap;
import io.jmix.mapsflowui.component.event.MapClickEvent;
import io.jmix.mapsflowui.component.model.feature.MarkerFeature;
import io.jmix.mapsflowui.component.model.source.DataVectorSource;
import io.jmix.mapsflowui.kit.component.model.style.Fill;
import io.jmix.mapsflowui.kit.component.model.style.Style;
import io.jmix.mapsflowui.kit.component.model.style.image.Anchor;
import io.jmix.mapsflowui.kit.component.model.style.image.CircleStyle;
import io.jmix.mapsflowui.kit.component.model.style.image.IconOrigin;
import io.jmix.mapsflowui.kit.component.model.style.image.IconStyle;
import io.jmix.mapsflowui.kit.component.model.style.stroke.Stroke;
import io.jmix.mapsflowui.kit.component.model.style.text.Padding;
import io.jmix.mapsflowui.kit.component.model.style.text.TextStyle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.Random;

@Route(value = "DashboardView", layout = MainView.class)
@ViewController("DashboardView")
@ViewDescriptor("dashboard-view.xml")
public class DashboardView extends StandardView {

    private static final Logger log = LoggerFactory.getLogger(DashboardView.class);

    @Autowired
    private VehiclesService vehiclesService;

    @ViewComponent
    private InstanceContainer<Tour> tourDc;
    @ViewComponent
    private CollectionContainer<Vehicle> vehiclesDc;
    @ViewComponent
    private CollectionLoader<Vehicle> vehiclesDl;

    @ViewComponent
    private GeoMap map;

    private Random random = new Random();

    @Subscribe
    public void onInit(final InitEvent event) {
        initVehicleLayer();
        initRouteLayer();
        initStopsLayer();
    }

    private void initRouteLayer() {
        DataVectorSource<Tour> routesSource = (DataVectorSource<Tour>) map.getLayer("routesLayer").getSource();
        routesSource.setStyleProvider(tour -> new Style()
                .withStroke(new Stroke()
                        .withWidth(4.)
                        .withColor("#188A00")));
    }

    private void initVehicleLayer() {
        DataVectorSource<Vehicle> vehiclesSource = (DataVectorSource<Vehicle>) map.getLayer("vehiclesLayer").getSource();
        vehiclesSource.addGeoObjectClickListener(clickEvent -> {
            Vehicle vehicle = clickEvent.getItem();

            // If the same tour, reset vehicles
            if (vehicle.getTour().equals(tourDc.getItemOrNull())) {
                tourDc.setItem(null);
                vehiclesDl.load();
            } else {
                tourDc.setItem(vehicle.getTour());
                vehiclesDc.getMutableItems().clear();
                vehiclesDc.getMutableItems().add(vehicle);
            }
        });

        StreamResource streamResource =
                new StreamResource("marker.png", () -> MarkerFeature.class
                        .getResourceAsStream("/META-INF/frontend/jmix-openlayers-map/icon/marker.png"));

        vehiclesSource.setStyleProvider(vehicle ->
                new Style()
                        .withImage(new IconStyle()
                                .withResource(streamResource)
                                .withScale(0.5)
                                .withAnchorOrigin(IconOrigin.BOTTOM_LEFT)
                                .withAnchor(new Anchor(0.49, 0.12)))
                        .withText(new TextStyle()
                                .withText(vehicle.getName())
                                .withFont("bold 15px sans-serif")
                                .withOffsetY(10)));
    }

    private void initStopsLayer() {
        DataVectorSource<TourStop> stopsSource = (DataVectorSource<TourStop>) map.getLayer("routeStopsLayer").getSource();
        stopsSource.setStyleProvider(tourStop ->
                new Style()
                        .withImage(new CircleStyle()
                                .withRadius(8)
                                .withStroke(new Stroke()
                                        .withWidth(1.)
                                        .withColor("#6566b8"))
                                .withFill(new Fill(generateRandomColor())))
                        .withText(new TextStyle()
                                .withPadding(new Padding(7, 7, 7, 7))
                                .withBackgroundFill(new Fill("rgba(255, 255, 255, 0.6)"))
                                .withOffsetY(30)
                                .withFont("bold 15px sans-serif")
                                .withText(tourStop.getName())));
    }

    @Subscribe("map")
    public void onMapMapClick(final MapClickEvent event) {
        log.info("TAAG " + event.getCoordinate());
    }

    @Install(to = "vehiclesDl", target = Target.DATA_LOADER)
    private List<Vehicle> vehiclesDlLoadDelegate(final LoadContext<Vehicle> loadContext) {
        return vehiclesService.loadVehicles();
    }

    private String generateRandomColor() {
        return String.format("#%06x", random.nextInt(0xffffff + 1));
    }
}