package com.company.tour.view.tourstop;

import com.company.tour.entity.TourStop;
import com.company.tour.view.main.MainView;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.HasValueAndElement;
import com.vaadin.flow.component.formlayout.FormLayout;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.router.Route;
import io.jmix.flowui.kit.action.ActionPerformedEvent;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.model.*;
import io.jmix.flowui.view.*;

@Route(value = "tourStops", layout = MainView.class)
@ViewController("TourStop.list")
@ViewDescriptor("tour-stop-list-view.xml")
@LookupComponent("tourStopsDataGrid")
@DialogMode(width = "64em")
public class TourStopListView extends StandardListView<TourStop> {

    @ViewComponent
    private DataContext dataContext;

    @ViewComponent
    private CollectionContainer<TourStop> tourStopsDc;

    @ViewComponent
    private InstanceContainer<TourStop> tourStopDc;

    @ViewComponent
    private InstanceLoader<TourStop> tourStopDl;

    @ViewComponent
    private VerticalLayout listLayout;

    @ViewComponent
    private FormLayout form;

    @ViewComponent
    private HorizontalLayout detailActions;

    @Subscribe
    public void onInit(final InitEvent event) {
        updateControls(false);
    }

    @Subscribe("tourStopsDataGrid.create")
    public void onTourStopsDataGridCreate(final ActionPerformedEvent event) {
        dataContext.clear();
        TourStop entity = dataContext.create(TourStop.class);
        tourStopDc.setItem(entity);
        updateControls(true);
    }

    @Subscribe("tourStopsDataGrid.edit")
    public void onTourStopsDataGridEdit(final ActionPerformedEvent event) {
        updateControls(true);
    }

    @Subscribe("saveBtn")
    public void onSaveButtonClick(final ClickEvent<JmixButton> event) {
        dataContext.save();
        tourStopsDc.replaceItem(tourStopDc.getItem());
        updateControls(false);
    }

    @Subscribe("cancelBtn")
    public void onCancelButtonClick(final ClickEvent<JmixButton> event) {
        dataContext.clear();
        tourStopDl.load();
        updateControls(false);
    }

    @Subscribe(id = "tourStopsDc", target = Target.DATA_CONTAINER)
    public void onTourStopsDcItemChange(final InstanceContainer.ItemChangeEvent<TourStop> event) {
        TourStop entity = event.getItem();
        dataContext.clear();
        if (entity != null) {
            tourStopDl.setEntityId(entity.getId());
            tourStopDl.load();
        } else {
            tourStopDl.setEntityId(null);
            tourStopDc.setItem(null);
        }
    }

    private void updateControls(boolean editing) {
        form.getChildren().forEach(component -> {
            if (component instanceof HasValueAndElement<?, ?> field) {
                field.setReadOnly(!editing);
            }
        });

        detailActions.setVisible(editing);
        listLayout.setEnabled(!editing);
    }
}